//-----------------------------------------------------------------------------
// Project:   Code for 'Using TStream to save application data' session
//            UK BUG Midlands Meeting 25/10/2002
// Module:    Class for streaming single object example
// Compiler:  Delphi 5+
// Author:    Mark Warriner                         mark@cybermunch.com
// Copyright: New Transducer Ltd. UK
//-----------------------------------------------------------------------------
unit MyClass;

interface

uses Classes, Windows, Clipbrd;

type
  TMyClass = class(TObject)
  public
    i: integer;
    x: extended;
    Str: String;
    procedure WriteToStream(S: TStream);
    procedure ReadFromStream(S: TStream);
  end;

var
  // Clipboard Format (see initialisation)
  CF_MyClassData: Cardinal = 0;

function WriteMyClassToClipboard(const S: TMemoryStream): Boolean;
function ReadMyClassFromClipboard(const S: TMemoryStream): Boolean;


implementation

//-----------------------------------------------------------------------------
procedure StreamWriteString(Stream: TStream; const S: String);
var
  L: Integer;
begin
  L:= Length(S);
  Stream.Write(L, SizeOf(L));
  if L > 0 then
    Stream.Write(S[1], L);
end;
{-----------------------------------------------------------------------------}
function StreamReadString(Stream: TStream): String;
var
  L: Integer;
begin
  Stream.Read(L, SizeOf(L));
  SetLength(Result, L);
  Stream.Read(Result[1], L);
end;
//-----------------------------------------------------------------------------
function WriteMyClassToClipboard(const S: TMemoryStream): Boolean;
var
  Data: THandle;
  DataPtr: Pointer;
begin
  Result:= false;
  Clipboard.Open;
try
  Data:= GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, S.Size);
  try
    DataPtr:= GlobalLock(Data);
    try
      System.Move(S.Memory^, DataPtr^, S.Size);
      Clipboard.Clear;
      SetClipboardData(CF_MyClassData, Data);
      Result:= true;
    finally
      GlobalUnlock(Data);
    end;
  except
    GlobalFree(Data);
    raise;
  end;
finally
  Clipboard.Close;
end; end;
//-----------------------------------------------------------------------------
function ReadMyClassFromClipboard(const S: TMemoryStream): Boolean;
var
  Data: THandle;
  DataPtr: Pointer;
begin
  Result:= false;
  Data:= 0;
  Clipboard.Open;
try
  Data:= GetClipboardData(CF_MyClassData);
  if Data <> 0 then begin
    DataPtr:= GlobalLock(Data);
    if Assigned(DataPtr) then begin
      S.WriteBuffer(DataPtr^, GlobalSize(Data));
      S.Position:= 0;
    end;
  end;
  Result:= true;
  GlobalUnlock(Data);
  Clipboard.Close;
except
  GlobalUnlock(Data);
  Clipboard.Close;
end; end;
//==============================================================================
{ TMyClass }
//==============================================================================
procedure TMyClass.ReadFromStream(S: TStream);
begin
  S.Read(i, 4);
  S.Read(x, 10);
  Str:= StreamReadString(S);
end;
//------------------------------------------------------------------------------
procedure TMyClass.WriteToStream(S: TStream);
begin
  S.Write(i, 4);
  S.Write(x, 10);
  StreamWriteString(S, Str);
end;
//------------------------------------------------------------------------------

initialization
  CF_MyClassData:= RegisterClipboardFormat('My Class Data');
end.
